import { Strudelapp } from './strudelapp';

const tmuxscript = `#!/bin/bash
#SBATCH -J tmux
tmux new-session -d -s $SLURM_JOB_NAME bash
# determine the process id of the tmux server
pid=$( /bin/ps x | /bin/grep -i "[t]mux new-session -d -s" | sed 's/^\ *//' | cut -f 1 -d " " )
ps x
# Sleep until the tmux server exits
while [ -e /proc/$pid ]; do sleep 5; done
`

const jupyterscript = `#!/bin/bash
#SBATCH -J Jupyter
/projects/pMOSP/chines/jupyter-venv/bin/jupyter-notebook
`

const stddesktop = `#!/bin/bash
#SBATCH --job-name=desktop
# --exclusive allows the job to consume all resources on the node reguardless of how many cpus/gpus/memory/etc
#SBATCH --exclusive
#SBATCH --nodes=1
#SBATCH --ntasks=3
#SBATCH --cpus-per-task=1
#SBATCH --gres=gpu:K1:1
#SBATCH --partition=m3f
#SBATCH --mem=10288M

#module purge
module load tigervnc/1.8.0
#module list
export PATH=$PATH:$XMASSIVESERVICEPATH/bin
export XDG_CONFIG_DIRS=$XMASSIVESERVICEPATH/xdg_config
export XDG_DATA_DIRS=$XMASSIVESERVICEPATH/xdg_data:/usr/share:/usr/local/share
echo " Starting VNC..."
#Remove old pid files that should not exist due to vncserver not being shut down cleanly last time
#rm -f ~/.vnc/$HOSTNAME:1.pid
#vncserver -xstartup $XSTARTUP #:1 so we can start multiple VNC servers
vncserver -xstartup /usr/local/desktop/services/massive-std/xstartup/xstartup
xinit /usr/bin/xterm -- -sharevts -novtswitch -config xorg.conf &
#vncserver
while true;
do
    sleep 30;
done`

const vncviewer = { 'cmd': ['python','vncviewer.py','--password','{password}','--host','localhost','--port','{localtunnelport}'], 'redir':'?password={password}'}

const tmuxclient = {'cmd': ['/usr/bin/gnome-terminal', '--','ssh','-t',
                            'localhost','-p','{localtunnelport}','-o',
                            'StrictHostkeyChecking=No','/bin/tmux a {sessionname}'],
                            'redir': null}


const jupyterclient = {'cmd': null, 'redir': '?token={token}'}

export const STRUDELAPPS: Strudelapp[] = [
  { url: null, name: 'tmux', startscript: tmuxscript,
    paramscmd: '/home/chines/smuxparams.py', client: tmuxclient, localbind: false },
  { url: null, name: 'Jupyter Notebook', startscript: jupyterscript,
    paramscmd: '/home/chines/jupyter_params.py', client: jupyterclient, localbind: true },
  { url: null, name: 'Standard Desktop', startscript: stddesktop,
    paramscmd: '/home/chines/desktop_params.py', client: vncviewer, localbind: false },
];
