import { Injectable } from '@angular/core';
import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';
import { catchError, map, tap } from 'rxjs/operators';
import { Job } from './job';
import {BehaviorSubject} from 'rxjs/BehaviorSubject';
import { Strudelapp } from './strudelapp';
import { Computesite } from './computesite';
import { Identity } from './identity';
import { BatchInterface} from './batchinterface';
import {ComputesitesService} from './computesites.service';
import { StrudelappsService } from './strudelapps.service';
import { timer } from 'rxjs/observable/timer';
import { repeat } from 'rxjs/operators';
import {LocationStrategy} from '@angular/common';
// import { keypair } from 'keypair';
import * as keypair from 'keypair';
import * as forge from "node-forge";
import { ActivatedRoute } from '@angular/router';

// import keypair = require('keypair');
// import forge = require('node-forge');


@Injectable()
export class TesService {
private Base='http://localhost:5000'
//private Base='https://vm-118-138-240-255.erc.monash.edu.au/tes'
public authorised: BehaviorSubject<boolean>;
public tesSelected: BehaviorSubject<boolean>;
public statusMsg: BehaviorSubject<any>;
public jobs: any[];
public busy: BehaviorSubject<boolean> ;
public testingAuth: BehaviorSubject<boolean>;
public joblist: BehaviorSubject<Job[]>;
private timerSubscription: any;
private token: string;
private signing_endpoint: string;
private sshauthzbase: string;
private sshauthz: string;
private client_id: string;
private keypair: any;
private ssh: any;
private sshcert: any;
private identities: Identity[];
private batchinterface: BatchInterface;



  constructor(private http: HttpClient,
              private locationStrategy: LocationStrategy,
              private route: ActivatedRoute,
              private computesite: ComputesitesService,
              private strudelappsService: StrudelappsService) {
    this.statusMsg = new BehaviorSubject<any>('');
    this.authorised = new BehaviorSubject<boolean>(false);
    this.tesSelected = new BehaviorSubject<boolean>(true);
    this.testingAuth = new BehaviorSubject<boolean>(false);
    this.busy = new BehaviorSubject<boolean>(false);
    this.joblist = new BehaviorSubject<Job[]>([]);
    this.timerSubscription = null;
    this.token = null;
    this.sshauthzbase = "https://autht.massive.org.au/hpcid/";
    this.sshauthz = this.sshauthzbase + 'oauth/authorize';
    this.signing_endpoint = this.sshauthzbase + 'api/v1/sign_key';
    this.client_id = "86c06039-e589-4b39-9d1f-9eca431be18f";
    this.route.fragment.subscribe(frag => this.getCert(frag));
    this.batchinterface = new BatchInterface();
    this.identities=[];
    this.batchinterface = { cancelcmd: 'scancel {jobid}', statcmd: '/home/chines/jsonstat.py',
                            submitcmd: 'sbatch --partition=m3f', internalfirewall: false }
 }


  updateJoblist(resp) {
    let joblist = <Job[]>resp;
    for (let j of joblist) {
      j.app = this.strudelappsService.getApp(j.name)
    }
    this.joblist.next(joblist);
    this.statusMsg.next(null);
  }

  submitted(resp: any ) {
    this.busy.next(false);
    this.statusMsg.next('Updating job list');
    this.getJobs();
  }

 buildParams(app: Strudelapp, identity: Identity, batchinterface: BatchInterface): string {
   let params = new URLSearchParams();
   let appstr = JSON.stringify(app);
   params.set('app',appstr);
   let interfacestr = JSON.stringify(batchinterface);
   params.set('interface',interfacestr);
   let identitystr = JSON.stringify(identity);
   params.set('identity',identitystr);
   return params.toString();
 }

 getJobs() {
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   this.statusMsg.next(null);
   for (let identity of this.identities) {
      let paramstr = this.buildParams(null,identity,this.batchinterface);
      this.http.get<Job[]>(this.Base+'/stat'+'?'+paramstr,options)
                      .pipe(catchError(this.handleError('getJobs',[])))
                      .subscribe(resp => this.updateJoblist(resp));
                    }
 }


 submit(app: Strudelapp, identity: Identity) {
   console.log("In tes submit url "+this.Base+'/submit');
   console.log("starting app"+app.name);
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   this.statusMsg.next('Submitting job');
   this.busy.next(true);
   let paramstr = this.buildParams(app,identity,this.batchinterface);
   this.http.post<any>(this.Base+'/submit'+'?'+paramstr,{}, options)
                                         .pipe(catchError(this.handleError('submit',[])))
                                         .subscribe(resp => this.busy.next(false));

 }

 cancel(job: Job) {
   console.log("In tes cancel");
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   this.statusMsg.next(null);
   let data = {};
   let paramstr = this.buildParams(job.app,job.identity,this.batchinterface);
   this.http.delete<any>(this.Base+'/cancel/'+job.jobid+'?'+paramstr, options)
                      .pipe(catchError(this.handleError('cancel',[])))
                      .subscribe(resp => this.submitted(resp));
 }

 public connect(job: Job) {
   this.statusMsg.next(null);
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   this.busy.next(true);
   let paramstr = this.buildParams(job.app,job.identity,this.batchinterface);
   let appwindow = window.open(this.Base+"/connect/"+job.jobid+"/"+job.batch_host+"?"+paramstr);
   appwindow.focus();
   this.busy.next(false);
 }



getCert(frag: string) {
   // Given a URL fragment, extract the authentication token, and use it to get
   // an SSH certificate. SSH cert to be used on subsequent calls to the TES
   console.log('in tesservice getCert argument is',frag);
   console.log('fragment.getvalue is')
   console.log(frag)
   let match = frag.match(/access_token\=([\S\s]*?)[&|$]/);
   if (match == null) {
     return;
   }
   this.authorised.next(true);
   console.log(match);
   this.token = match[1];
   console.log(this.token);
   // this.token = frag;
   console.log('tes stored the token'+this.token);

   // let data = {'token': this.token, 'sshauthz_endpoint': this.signing_endpoint}
   // console.log(data);
   console.log("Generating key matter")
   let starttime = new Date();
   this.keypair = keypair();
   let publicKey = forge.pki.publicKeyFromPem(this.keypair.public);
   this.ssh = forge.ssh.publicKeyToOpenSSH(publicKey, 'sv2@monash.edu');
   let endtime = new Date();
   console.log("generating new keys took", endtime.valueOf() - starttime.valueOf())

   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};

   let data = {'token': this.token, 'pubkey': this.ssh, 'signing_url': this.signing_endpoint};

   this.busy.next(true);
   this.statusMsg.next("Generating Certificates ...")
   console.log('posting to getcert',this.Base);
   this.http.post<any>(this.Base+'/getcert',data, options)
                      .pipe(catchError(this.handleError('getCert',[])))
                      .subscribe(resp => this.storeCert(resp));
}

private storeCert(resp) {
  console.log(resp);
  this.busy.next(false);
  this.authorised.next(true);
  this.testingAuth.next(false);

  console.log(resp['cert']);
  this.sshcert = resp['cert'];
  let headers = new HttpHeaders();
  let options = { headers: headers, withCredentials: true};

  this.statusMsg.next("Authorising ...")
  let data = {'key': this.keypair.private, 'cert': this.sshcert};

  this.http.post<any>(this.Base+'/sshagent',data,options)
                      .pipe(catchError(this.handleError('storeCert',[])))
                      .subscribe(resp => this.getIdentities(resp));
}

private getIdentities(resp) {
  this.statusMsg.next("Updating the list of available accounts")
  this
  let headers = new HttpHeaders();
  let options = { headers: headers, withCredentials: true};
  this.http.get<any>(this.Base+'/sshagent',options)
                    .pipe(catchError(this.handleError('updateIdentities',[])))
                    .subscribe(resp => this.updateIdentities(resp));
}

private killAgent() {
  this.statusMsg.next("Updating the list of available accounts")
  this
  let headers = new HttpHeaders();
  let options = { headers: headers, withCredentials: true};
  this.http.delete<any>(this.Base+'/sshagent',options)
                    .pipe(catchError(this.handleError('updateIdentities',[])))
                    .subscribe(resp => this.updateIdentities(resp));
}

private updateIdentities(resp) {
  let certcontents = resp;
  console.log(certcontents);
  this.identities = [];
  console.log('identities updated');
  let sites = this.computesite.getComputeSites();
  for (let i in certcontents) {
    let id = new Identity()
    id.username = certcontents[i].Principals[0];
    id.site = sites[0]
    this.identities.push(id)
  }
  if (this.identities.length == 0) {
    this.statusMsg.next(null);
    return;
  }
  console.log(this.identities);
  this.statusMsg.next('Updating Job list');
  this.startPolling();
}

private startPolling() {
  this.statusMsg.next(null);
  if (!(this.timerSubscription === null)) {
    console.log('unsubscribing timer');
    this.timerSubscription.unsubscribe()
  }
  console.log('creating timer to poll for jobs');
  // this.timerSubscription = timer(100,5000).subscribe(() => this.getJobs());

  this.timerSubscription = timer(5000).pipe(repeat()).subscribe(() => this.getJobs());
  this.getJobs();
}

public login() {
  let redirect_uri = window.location.origin+this.locationStrategy.getBaseHref()+"sshauthz_callback";
  let state="asdfzxcv";
  window.location.assign(this.sshauthz+"?response_type=token&redirect_uri="+redirect_uri+"&state="+state+"&client_id="+this.client_id);
}

public logout(): Boolean {
  this.token = null;
  this.sshcert = null;
  this.statusMsg.next(null);
  this.authorised.next(false);
  this.killAgent();
  if (!(this.timerSubscription === null)) {
    this.timerSubscription.unsubscribe()
  }
  return true;
}


public getstatusMsgSubject(): BehaviorSubject<any> {
  return this.statusMsg;
}


private handleError<T> (operation = 'operation', result?: T) {
  return (error: any): Observable<T> => {
    // TODO: send the error to remote logging infrastructure
    console.error(error); // log to console instead
    this.authorised.next(false);
    this.token = null;
    this.sshcert = null;

    if ( operation == 'getJobs') {
      this.statusMsg.next("Hmm, that didn't work. If you're using a local connection, please make sure Strudel-TES is running.");
    }
    if (operation == 'submit') {
      this.statusMsg.next("Hmm, I couldn't submit that job")
    }
    return of(result as T);
  };
}
}
